# Eneroth Merge Layers

# Copyright Julia Christina Eneroth, eneroth3@gmail.com

module EneMergeLayers

  def self.traverse_entities(entities, source_layer, target_layer)
    entities.each do |entity|
      entity.layer = target_layer if entity.layer == source_layer
    end

    nil
  end

  def self.perform_merge(source_layer, target_layer)
    model = Sketchup.active_model
    model.definitions.each { |d| traverse_entities(d.entities, source_layer, target_layer) }
    traverse_entities(model.entities, source_layer, target_layer)
    model.layers.remove(source_layer) # SU2015 and later.

    nil
  end

  def self.merge_layers
    model = Sketchup.active_model
    layers = model.layers
    if layers.size == 1
      UI.messagebox("There are no layers to merge in model.")
      return
    end

    # nil layer ("Layer0") is excluded from first list.
    prompts = ["Merge layer", "into layer"]
    defaults = [layers.drop(1).sort.first.name, layers.first.name]
    list = [layers.drop(1).map(&:name).sort.join("|"), layers.map(&:name).sort.join("|")]
    input = UI.inputbox(prompts, defaults, list, "Eneroth Layer Merger")
    return unless input
    source_layer, target_layer = (input.map { |i| layers[i] })
    if source_layer == target_layer
      return
    end

    model.start_operation("Purge Layer", true)
    msg =
      "Notify if layer '#{source_layer.name}' appears in this model in the future, "\
      "e.g. when pasting from another model?"
    if UI.messagebox(msg, MB_YESNO) == IDYES
      add_layer_listener(source_layer, target_layer)
    end
    perform_merge(source_layer, target_layer)
    model.commit_operation

    nil
  end

  def self.add_layer_listener(source, target)
    model = Sketchup.active_model
    to_merge = Hash[*model.layers.get_attribute(ID, "to_merge", []).flatten]
    to_merge[source.name] = target.name
    model.layers.set_attribute(ID, "to_merge", to_merge.to_a)

    nil
  end

  def self.remove_layer_listener(source)
    model = Sketchup.active_model
    to_merge = Hash[*model.layers.get_attribute(ID, "to_merge", []).flatten]
    to_merge.delete(source.name)
    model.layers.set_attribute(ID, "to_merge", to_merge.to_a)
  end

  def self.get_saved_layer_purge_target(layer)
    model = Sketchup.active_model
    to_merge = Hash[*model.layers.get_attribute(ID, "to_merge", []).flatten]
    name = to_merge[layer.name]

    name && model.layers[name]
  end

  def self.on_add_layer(new_layer)
    model = Sketchup.active_model

    target = get_saved_layer_purge_target(new_layer)
    return unless target

    msg =
      "Previously purged layer '#{new_layer.name}' has been re-added to the model. "\
      "Do you want to purge it into layer '#{target.name}' again? \n\n"\
      "Click No to not be notified about this layer again."
    case UI.messagebox(msg, MB_YESNO)
    when IDYES
      model.start_operation("Purge Layer", true)
      perform_merge(new_layer, target)
      model.commit_operation
    when IDNO
      model.start_operation("Change Layer Purging Property", true)
      remove_layer_listener(new_layer)
      model.commit_operation
    end

    nil
  end

  module Observers

    class MyLayersObserver < Sketchup::LayersObserver
      def onLayerAdded(_, layer)
        EneMergeLayers.on_add_layer(layer)
      end
    end
    class MyAppObserver < Sketchup::AppObserver
      def onNewModel(model)
        model.layers.add_observer(MyLayersObserver.new)
      end
      def onOpenModel(model)
        model.layers.add_observer(MyLayersObserver.new)
      end
      def expectsStartupModelNotifications
        return false
      end
    end
    Sketchup.add_observer(MyAppObserver.new)
    Sketchup.active_model.layers.add_observer(MyLayersObserver.new)

  end

  menu = UI.menu("Plugins")
  item = menu.add_item(NAME + "...") { merge_layers }

end
